# coding: utf-8

"""
    Rightfoot Student Loan Repayment

    This API provides student loan repayment. You can find out more about Rightfoot at [docs.rightfoot.com](https://docs.rightfoot.com).   # noqa: E501

    OpenAPI spec version: 1.2.0
    Contact: engineering@rightfoot.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class CreateFundingSourceRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'beneficiary_uuid': 'str',
        'email_address': 'str',
        'plaid_public_token': 'str'
    }

    attribute_map = {
        'beneficiary_uuid': 'beneficiaryUuid',
        'email_address': 'emailAddress',
        'plaid_public_token': 'plaidPublicToken'
    }

    def __init__(self, beneficiary_uuid=None, email_address=None, plaid_public_token=None):  # noqa: E501
        """CreateFundingSourceRequest - a model defined in Swagger"""  # noqa: E501
        self._beneficiary_uuid = None
        self._email_address = None
        self._plaid_public_token = None
        self.discriminator = None
        if beneficiary_uuid is not None:
            self.beneficiary_uuid = beneficiary_uuid
        if email_address is not None:
            self.email_address = email_address
        if plaid_public_token is not None:
            self.plaid_public_token = plaid_public_token

    @property
    def beneficiary_uuid(self):
        """Gets the beneficiary_uuid of this CreateFundingSourceRequest.  # noqa: E501

        Unique identifier generated by Rightfoot for beneficiary.  # noqa: E501

        :return: The beneficiary_uuid of this CreateFundingSourceRequest.  # noqa: E501
        :rtype: str
        """
        return self._beneficiary_uuid

    @beneficiary_uuid.setter
    def beneficiary_uuid(self, beneficiary_uuid):
        """Sets the beneficiary_uuid of this CreateFundingSourceRequest.

        Unique identifier generated by Rightfoot for beneficiary.  # noqa: E501

        :param beneficiary_uuid: The beneficiary_uuid of this CreateFundingSourceRequest.  # noqa: E501
        :type: str
        """

        self._beneficiary_uuid = beneficiary_uuid

    @property
    def email_address(self):
        """Gets the email_address of this CreateFundingSourceRequest.  # noqa: E501

        Valid email address, which can be used to reach the user. Rightfoot and its partners do not conduct regular communication with end-users. Regulations require bank partner to have the ability to contact the end-user.   # noqa: E501

        :return: The email_address of this CreateFundingSourceRequest.  # noqa: E501
        :rtype: str
        """
        return self._email_address

    @email_address.setter
    def email_address(self, email_address):
        """Sets the email_address of this CreateFundingSourceRequest.

        Valid email address, which can be used to reach the user. Rightfoot and its partners do not conduct regular communication with end-users. Regulations require bank partner to have the ability to contact the end-user.   # noqa: E501

        :param email_address: The email_address of this CreateFundingSourceRequest.  # noqa: E501
        :type: str
        """

        self._email_address = email_address

    @property
    def plaid_public_token(self):
        """Gets the plaid_public_token of this CreateFundingSourceRequest.  # noqa: E501

        Public token provided by Plaid Link representing the beneficiary’s bank account. Grants Rightfoot permission to draw from the account backing this token.   # noqa: E501

        :return: The plaid_public_token of this CreateFundingSourceRequest.  # noqa: E501
        :rtype: str
        """
        return self._plaid_public_token

    @plaid_public_token.setter
    def plaid_public_token(self, plaid_public_token):
        """Sets the plaid_public_token of this CreateFundingSourceRequest.

        Public token provided by Plaid Link representing the beneficiary’s bank account. Grants Rightfoot permission to draw from the account backing this token.   # noqa: E501

        :param plaid_public_token: The plaid_public_token of this CreateFundingSourceRequest.  # noqa: E501
        :type: str
        """

        self._plaid_public_token = plaid_public_token

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(CreateFundingSourceRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CreateFundingSourceRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Rightfoot Student Loan Repayment

    This API provides student loan repayment. You can find out more about Rightfoot at [docs.rightfoot.com](https://docs.rightfoot.com).   # noqa: E501

    OpenAPI spec version: 1.2.0
    Contact: engineering@rightfoot.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class AddGenericLoanToBeneficiaryRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'uuid': 'str',
        'plaid_institution_id': 'str',
        'account_number': 'str',
        'sequence_number': 'str',
        'payment_reference_number': 'str',
        'guarantor': 'str'
    }

    attribute_map = {
        'uuid': 'uuid',
        'plaid_institution_id': 'plaidInstitutionId',
        'account_number': 'accountNumber',
        'sequence_number': 'sequenceNumber',
        'payment_reference_number': 'paymentReferenceNumber',
        'guarantor': 'guarantor'
    }

    def __init__(self, uuid=None, plaid_institution_id=None, account_number=None, sequence_number=None, payment_reference_number=None, guarantor=None):  # noqa: E501
        """AddGenericLoanToBeneficiaryRequest - a model defined in Swagger"""  # noqa: E501
        self._uuid = None
        self._plaid_institution_id = None
        self._account_number = None
        self._sequence_number = None
        self._payment_reference_number = None
        self._guarantor = None
        self.discriminator = None
        if uuid is not None:
            self.uuid = uuid
        if plaid_institution_id is not None:
            self.plaid_institution_id = plaid_institution_id
        if account_number is not None:
            self.account_number = account_number
        if sequence_number is not None:
            self.sequence_number = sequence_number
        if payment_reference_number is not None:
            self.payment_reference_number = payment_reference_number
        if guarantor is not None:
            self.guarantor = guarantor

    @property
    def uuid(self):
        """Gets the uuid of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501

        Unique identifier generated by Rightfoot.  # noqa: E501

        :return: The uuid of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501
        :rtype: str
        """
        return self._uuid

    @uuid.setter
    def uuid(self, uuid):
        """Sets the uuid of this AddGenericLoanToBeneficiaryRequest.

        Unique identifier generated by Rightfoot.  # noqa: E501

        :param uuid: The uuid of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501
        :type: str
        """

        self._uuid = uuid

    @property
    def plaid_institution_id(self):
        """Gets the plaid_institution_id of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501

        Unique identifier used by Plaid for the loan servicer institution.   # noqa: E501

        :return: The plaid_institution_id of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501
        :rtype: str
        """
        return self._plaid_institution_id

    @plaid_institution_id.setter
    def plaid_institution_id(self, plaid_institution_id):
        """Sets the plaid_institution_id of this AddGenericLoanToBeneficiaryRequest.

        Unique identifier used by Plaid for the loan servicer institution.   # noqa: E501

        :param plaid_institution_id: The plaid_institution_id of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501
        :type: str
        """

        self._plaid_institution_id = plaid_institution_id

    @property
    def account_number(self):
        """Gets the account_number of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501

        The root account number for the loan.  # noqa: E501

        :return: The account_number of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501
        :rtype: str
        """
        return self._account_number

    @account_number.setter
    def account_number(self, account_number):
        """Sets the account_number of this AddGenericLoanToBeneficiaryRequest.

        The root account number for the loan.  # noqa: E501

        :param account_number: The account_number of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501
        :type: str
        """

        self._account_number = account_number

    @property
    def sequence_number(self):
        """Gets the sequence_number of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501

        If available, the sub-loan sequence number within the account.   # noqa: E501

        :return: The sequence_number of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501
        :rtype: str
        """
        return self._sequence_number

    @sequence_number.setter
    def sequence_number(self, sequence_number):
        """Sets the sequence_number of this AddGenericLoanToBeneficiaryRequest.

        If available, the sub-loan sequence number within the account.   # noqa: E501

        :param sequence_number: The sequence_number of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501
        :type: str
        """

        self._sequence_number = sequence_number

    @property
    def payment_reference_number(self):
        """Gets the payment_reference_number of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501

        If a different payment reference number is available for accounts, identifies the loan account for payment purposes. Often the same as accountNumber but can be specified when known to be different.   # noqa: E501

        :return: The payment_reference_number of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501
        :rtype: str
        """
        return self._payment_reference_number

    @payment_reference_number.setter
    def payment_reference_number(self, payment_reference_number):
        """Sets the payment_reference_number of this AddGenericLoanToBeneficiaryRequest.

        If a different payment reference number is available for accounts, identifies the loan account for payment purposes. Often the same as accountNumber but can be specified when known to be different.   # noqa: E501

        :param payment_reference_number: The payment_reference_number of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501
        :type: str
        """

        self._payment_reference_number = payment_reference_number

    @property
    def guarantor(self):
        """Gets the guarantor of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501

        The name of the guarantor of the loan if used at the loan servicer.   # noqa: E501

        :return: The guarantor of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501
        :rtype: str
        """
        return self._guarantor

    @guarantor.setter
    def guarantor(self, guarantor):
        """Sets the guarantor of this AddGenericLoanToBeneficiaryRequest.

        The name of the guarantor of the loan if used at the loan servicer.   # noqa: E501

        :param guarantor: The guarantor of this AddGenericLoanToBeneficiaryRequest.  # noqa: E501
        :type: str
        """

        self._guarantor = guarantor

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(AddGenericLoanToBeneficiaryRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AddGenericLoanToBeneficiaryRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Rightfoot Student Loan Repayment

    This API provides student loan repayment. You can find out more about Rightfoot at [docs.rightfoot.com](https://docs.rightfoot.com).   # noqa: E501

    OpenAPI spec version: 1.2.0
    Contact: engineering@rightfoot.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class CreatePaymentRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'beneficiary_uuid': 'str',
        'payment_amount': 'int',
        'loan_uuid': 'str',
        'funding_source_uuid': 'str'
    }

    attribute_map = {
        'beneficiary_uuid': 'beneficiaryUuid',
        'payment_amount': 'paymentAmount',
        'loan_uuid': 'loanUuid',
        'funding_source_uuid': 'fundingSourceUuid'
    }

    def __init__(self, beneficiary_uuid=None, payment_amount=None, loan_uuid=None, funding_source_uuid=None):  # noqa: E501
        """CreatePaymentRequest - a model defined in Swagger"""  # noqa: E501
        self._beneficiary_uuid = None
        self._payment_amount = None
        self._loan_uuid = None
        self._funding_source_uuid = None
        self.discriminator = None
        if beneficiary_uuid is not None:
            self.beneficiary_uuid = beneficiary_uuid
        if payment_amount is not None:
            self.payment_amount = payment_amount
        if loan_uuid is not None:
            self.loan_uuid = loan_uuid
        if funding_source_uuid is not None:
            self.funding_source_uuid = funding_source_uuid

    @property
    def beneficiary_uuid(self):
        """Gets the beneficiary_uuid of this CreatePaymentRequest.  # noqa: E501

        Unique identifier generated by Rightfoot for beneficiary.  # noqa: E501

        :return: The beneficiary_uuid of this CreatePaymentRequest.  # noqa: E501
        :rtype: str
        """
        return self._beneficiary_uuid

    @beneficiary_uuid.setter
    def beneficiary_uuid(self, beneficiary_uuid):
        """Sets the beneficiary_uuid of this CreatePaymentRequest.

        Unique identifier generated by Rightfoot for beneficiary.  # noqa: E501

        :param beneficiary_uuid: The beneficiary_uuid of this CreatePaymentRequest.  # noqa: E501
        :type: str
        """

        self._beneficiary_uuid = beneficiary_uuid

    @property
    def payment_amount(self):
        """Gets the payment_amount of this CreatePaymentRequest.  # noqa: E501

        Amount to pay to beneficiary's student loans, in cents USD.   # noqa: E501

        :return: The payment_amount of this CreatePaymentRequest.  # noqa: E501
        :rtype: int
        """
        return self._payment_amount

    @payment_amount.setter
    def payment_amount(self, payment_amount):
        """Sets the payment_amount of this CreatePaymentRequest.

        Amount to pay to beneficiary's student loans, in cents USD.   # noqa: E501

        :param payment_amount: The payment_amount of this CreatePaymentRequest.  # noqa: E501
        :type: int
        """

        self._payment_amount = payment_amount

    @property
    def loan_uuid(self):
        """Gets the loan_uuid of this CreatePaymentRequest.  # noqa: E501

        UUID of the Loan to make payment against if created generically.   # noqa: E501

        :return: The loan_uuid of this CreatePaymentRequest.  # noqa: E501
        :rtype: str
        """
        return self._loan_uuid

    @loan_uuid.setter
    def loan_uuid(self, loan_uuid):
        """Sets the loan_uuid of this CreatePaymentRequest.

        UUID of the Loan to make payment against if created generically.   # noqa: E501

        :param loan_uuid: The loan_uuid of this CreatePaymentRequest.  # noqa: E501
        :type: str
        """

        self._loan_uuid = loan_uuid

    @property
    def funding_source_uuid(self):
        """Gets the funding_source_uuid of this CreatePaymentRequest.  # noqa: E501

        UUID of the FundingSource to make payments from. This replaces any default funding source configured for the API application. If no default application funding source is specified, this new field is required. This is typical for applications, where the end-user (beneficiary) is making payments through their app.   # noqa: E501

        :return: The funding_source_uuid of this CreatePaymentRequest.  # noqa: E501
        :rtype: str
        """
        return self._funding_source_uuid

    @funding_source_uuid.setter
    def funding_source_uuid(self, funding_source_uuid):
        """Sets the funding_source_uuid of this CreatePaymentRequest.

        UUID of the FundingSource to make payments from. This replaces any default funding source configured for the API application. If no default application funding source is specified, this new field is required. This is typical for applications, where the end-user (beneficiary) is making payments through their app.   # noqa: E501

        :param funding_source_uuid: The funding_source_uuid of this CreatePaymentRequest.  # noqa: E501
        :type: str
        """

        self._funding_source_uuid = funding_source_uuid

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(CreatePaymentRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CreatePaymentRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Rightfoot Student Loan Repayment

    This API provides student loan repayment. You can find out more about Rightfoot at [docs.rightfoot.com](https://docs.rightfoot.com).   # noqa: E501

    OpenAPI spec version: 1.2.0
    Contact: engineering@rightfoot.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class MailingAddress(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'line1': 'str',
        'line2': 'str',
        'city': 'str',
        'state': 'str',
        'zip_code': 'str'
    }

    attribute_map = {
        'line1': 'line1',
        'line2': 'line2',
        'city': 'city',
        'state': 'state',
        'zip_code': 'zipCode'
    }

    def __init__(self, line1=None, line2=None, city=None, state=None, zip_code=None):  # noqa: E501
        """MailingAddress - a model defined in Swagger"""  # noqa: E501
        self._line1 = None
        self._line2 = None
        self._city = None
        self._state = None
        self._zip_code = None
        self.discriminator = None
        if line1 is not None:
            self.line1 = line1
        if line2 is not None:
            self.line2 = line2
        if city is not None:
            self.city = city
        if state is not None:
            self.state = state
        if zip_code is not None:
            self.zip_code = zip_code

    @property
    def line1(self):
        """Gets the line1 of this MailingAddress.  # noqa: E501

        Mandatory mailing address line.  # noqa: E501

        :return: The line1 of this MailingAddress.  # noqa: E501
        :rtype: str
        """
        return self._line1

    @line1.setter
    def line1(self, line1):
        """Sets the line1 of this MailingAddress.

        Mandatory mailing address line.  # noqa: E501

        :param line1: The line1 of this MailingAddress.  # noqa: E501
        :type: str
        """

        self._line1 = line1

    @property
    def line2(self):
        """Gets the line2 of this MailingAddress.  # noqa: E501

        Optional second line of an address.  # noqa: E501

        :return: The line2 of this MailingAddress.  # noqa: E501
        :rtype: str
        """
        return self._line2

    @line2.setter
    def line2(self, line2):
        """Sets the line2 of this MailingAddress.

        Optional second line of an address.  # noqa: E501

        :param line2: The line2 of this MailingAddress.  # noqa: E501
        :type: str
        """

        self._line2 = line2

    @property
    def city(self):
        """Gets the city of this MailingAddress.  # noqa: E501

        The city of the location.  # noqa: E501

        :return: The city of this MailingAddress.  # noqa: E501
        :rtype: str
        """
        return self._city

    @city.setter
    def city(self, city):
        """Sets the city of this MailingAddress.

        The city of the location.  # noqa: E501

        :param city: The city of this MailingAddress.  # noqa: E501
        :type: str
        """

        self._city = city

    @property
    def state(self):
        """Gets the state of this MailingAddress.  # noqa: E501

        The two-letter state/territory abbreviation.  # noqa: E501

        :return: The state of this MailingAddress.  # noqa: E501
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this MailingAddress.

        The two-letter state/territory abbreviation.  # noqa: E501

        :param state: The state of this MailingAddress.  # noqa: E501
        :type: str
        """

        self._state = state

    @property
    def zip_code(self):
        """Gets the zip_code of this MailingAddress.  # noqa: E501

        US Zone Improvement Plan (ZIP) Code, or alternatively ZIP+4 Code.   # noqa: E501

        :return: The zip_code of this MailingAddress.  # noqa: E501
        :rtype: str
        """
        return self._zip_code

    @zip_code.setter
    def zip_code(self, zip_code):
        """Sets the zip_code of this MailingAddress.

        US Zone Improvement Plan (ZIP) Code, or alternatively ZIP+4 Code.   # noqa: E501

        :param zip_code: The zip_code of this MailingAddress.  # noqa: E501
        :type: str
        """

        self._zip_code = zip_code

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(MailingAddress, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MailingAddress):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

# flake8: noqa

"""
    Rightfoot Student Loan Repayment

    This API provides student loan repayment. You can find out more about Rightfoot at [docs.rightfoot.com](https://docs.rightfoot.com).   # noqa: E501

    OpenAPI spec version: 1.2.0
    Contact: engineering@rightfoot.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

# import apis into sdk package
from rightfoot.api.rightfoot_api import RightfootApi
# import ApiClient
from rightfoot.api_client import ApiClient
from rightfoot.configuration import Configuration
# import models into sdk package
from rightfoot.models.add_generic_loan_to_beneficiary_request import AddGenericLoanToBeneficiaryRequest
from rightfoot.models.add_generic_loan_to_beneficiary_response import AddGenericLoanToBeneficiaryResponse
from rightfoot.models.add_plaid_token_to_beneficiary_request import AddPlaidTokenToBeneficiaryRequest
from rightfoot.models.add_plaid_token_to_beneficiary_response import AddPlaidTokenToBeneficiaryResponse
from rightfoot.models.beneficiary import Beneficiary
from rightfoot.models.create_beneficiary_request import CreateBeneficiaryRequest
from rightfoot.models.create_beneficiary_response import CreateBeneficiaryResponse
from rightfoot.models.create_funding_source_request import CreateFundingSourceRequest
from rightfoot.models.create_funding_source_response import CreateFundingSourceResponse
from rightfoot.models.create_payment_request import CreatePaymentRequest
from rightfoot.models.create_payment_response import CreatePaymentResponse
from rightfoot.models.funding_source import FundingSource
from rightfoot.models.loan import Loan
from rightfoot.models.mailing_address import MailingAddress
from rightfoot.models.payment import Payment



# coding: utf-8

"""
    Rightfoot Student Loan Repayment

    This API provides student loan repayment. You can find out more about
    Rightfoot at [docs.rightfoot.com](https://docs.rightfoot.com).

    OpenAPI spec version: 1.1.0
    Contact: engineering@rightfoot.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from setuptools import setup, find_packages  # noqa: H301
from os import path

NAME = "rightfoot"
VERSION = "1.2.0a1"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = ["urllib3 >= 1.15", "six >= 1.10", "certifi", "python-dateutil"]

def read_readme():
  """Read the contents of the README file."""
  this_directory = path.abspath(path.dirname(__file__))
  with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
      return f.read()


setup(
    name=NAME,
    version=VERSION,
    description="Python API client for Rightfoot student debt repayment.",
    author_email="engineering@rightfoot.com",
    url="https://rightfoot.com",
    keywords=[
        "Rightfoot Student Loan Repayment",
        "rightfoot",
        "student",
        "debt",
        "payments",
    ],
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    long_description=read_readme(),
    long_description_content_type="text/markdown"
)



# coding: utf-8

"""
    Rightfoot Student Loan Repayment

    This API provides student loan repayment. You can find out more about Rightfoot at [docs.rightfoot.com](https://docs.rightfoot.com).   # noqa: E501

    OpenAPI spec version: 1.2.0
    Contact: engineering@rightfoot.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from rightfoot.api_client import ApiClient


class RightfootApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def add_generic_loan_to_beneficiary(self, body, **kwargs):  # noqa: E501
        """Links generic loan to beneficiary.  # noqa: E501

        NOTE: ALPHA.\\nNOTE: Currently only supports data sourced from Plaid. More generic support for other sources will be added in the future.\\n Links a loan generically to a beneficiary. This is useful for API applications, which already have loan data through their own data aggregation mechanisms and don’t need to offload and isolate the processing of that data to Rightfoot using Plaid tokens. This adds the requisite data for processing a loan payment. This does not perform any validation on the loan data like Plaid does by accessing that information directly, so data cleanliness is left to the API application.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_generic_loan_to_beneficiary(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param AddGenericLoanToBeneficiaryRequest body: (required)
        :return: AddGenericLoanToBeneficiaryResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_generic_loan_to_beneficiary_with_http_info(body, **kwargs)  # noqa: E501
        else:
            (data) = self.add_generic_loan_to_beneficiary_with_http_info(body, **kwargs)  # noqa: E501
            return data

    def add_generic_loan_to_beneficiary_with_http_info(self, body, **kwargs):  # noqa: E501
        """Links generic loan to beneficiary.  # noqa: E501

        NOTE: ALPHA.\\nNOTE: Currently only supports data sourced from Plaid. More generic support for other sources will be added in the future.\\n Links a loan generically to a beneficiary. This is useful for API applications, which already have loan data through their own data aggregation mechanisms and don’t need to offload and isolate the processing of that data to Rightfoot using Plaid tokens. This adds the requisite data for processing a loan payment. This does not perform any validation on the loan data like Plaid does by accessing that information directly, so data cleanliness is left to the API application.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_generic_loan_to_beneficiary_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param AddGenericLoanToBeneficiaryRequest body: (required)
        :return: AddGenericLoanToBeneficiaryResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_generic_loan_to_beneficiary" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `add_generic_loan_to_beneficiary`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['apiKey']  # noqa: E501

        return self.api_client.call_api(
            '/beneficiaries/addGenericLoan', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='AddGenericLoanToBeneficiaryResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def add_plaid_token_to_beneficiary(self, body, **kwargs):  # noqa: E501
        """Links Plaid loan to beneficiary.  # noqa: E501

        This endpoint connects an authenticated Plaid account using our Plaid public key 1773c898604f2bd4d68ecbbc8504f9. Rightfoot binds the loan accounts, exchanging the public key in the background, maintaining loan records.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_plaid_token_to_beneficiary(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param AddPlaidTokenToBeneficiaryRequest body: (required)
        :return: AddPlaidTokenToBeneficiaryResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.add_plaid_token_to_beneficiary_with_http_info(body, **kwargs)  # noqa: E501
        else:
            (data) = self.add_plaid_token_to_beneficiary_with_http_info(body, **kwargs)  # noqa: E501
            return data

    def add_plaid_token_to_beneficiary_with_http_info(self, body, **kwargs):  # noqa: E501
        """Links Plaid loan to beneficiary.  # noqa: E501

        This endpoint connects an authenticated Plaid account using our Plaid public key 1773c898604f2bd4d68ecbbc8504f9. Rightfoot binds the loan accounts, exchanging the public key in the background, maintaining loan records.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.add_plaid_token_to_beneficiary_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param AddPlaidTokenToBeneficiaryRequest body: (required)
        :return: AddPlaidTokenToBeneficiaryResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method add_plaid_token_to_beneficiary" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `add_plaid_token_to_beneficiary`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['apiKey']  # noqa: E501

        return self.api_client.call_api(
            '/beneficiaries/addPlaidToken', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='AddPlaidTokenToBeneficiaryResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def create_beneficiary(self, body, **kwargs):  # noqa: E501
        """This endpoint creates a new beneficiary.  # noqa: E501

        This endpoint creates a new beneficiary.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_beneficiary(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param CreateBeneficiaryRequest body: (required)
        :return: CreateBeneficiaryResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_beneficiary_with_http_info(body, **kwargs)  # noqa: E501
        else:
            (data) = self.create_beneficiary_with_http_info(body, **kwargs)  # noqa: E501
            return data

    def create_beneficiary_with_http_info(self, body, **kwargs):  # noqa: E501
        """This endpoint creates a new beneficiary.  # noqa: E501

        This endpoint creates a new beneficiary.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_beneficiary_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param CreateBeneficiaryRequest body: (required)
        :return: CreateBeneficiaryResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_beneficiary" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `create_beneficiary`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['apiKey']  # noqa: E501

        return self.api_client.call_api(
            '/beneficiaries', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='CreateBeneficiaryResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def create_funding_source(self, body, **kwargs):  # noqa: E501
        """Binds funding source for beneficiary making direct payments.  # noqa: E501

        Creates a new funding source attached to a beneficiary. Accepts a Plaid public token linked using Plaid Link in the API application. This provides instant verification of the account and avoids exposure of end-user financial data to API Applications and Rightfoot. This funding source can be specified to the payments API to override the default application funding source, if configured.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_funding_source(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param CreateFundingSourceRequest body: (required)
        :return: CreateFundingSourceResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_funding_source_with_http_info(body, **kwargs)  # noqa: E501
        else:
            (data) = self.create_funding_source_with_http_info(body, **kwargs)  # noqa: E501
            return data

    def create_funding_source_with_http_info(self, body, **kwargs):  # noqa: E501
        """Binds funding source for beneficiary making direct payments.  # noqa: E501

        Creates a new funding source attached to a beneficiary. Accepts a Plaid public token linked using Plaid Link in the API application. This provides instant verification of the account and avoids exposure of end-user financial data to API Applications and Rightfoot. This funding source can be specified to the payments API to override the default application funding source, if configured.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_funding_source_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param CreateFundingSourceRequest body: (required)
        :return: CreateFundingSourceResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_funding_source" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `create_funding_source`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['apiKey']  # noqa: E501

        return self.api_client.call_api(
            '/fundingSources', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='CreateFundingSourceResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def create_payment(self, body, **kwargs):  # noqa: E501
        """Issues a loan payment.  # noqa: E501

        This endpoint issues a request to transfer funds from the consuming application's funding source to the beneficiary's loan account.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_payment(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param CreatePaymentRequest body: (required)
        :return: CreatePaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_payment_with_http_info(body, **kwargs)  # noqa: E501
        else:
            (data) = self.create_payment_with_http_info(body, **kwargs)  # noqa: E501
            return data

    def create_payment_with_http_info(self, body, **kwargs):  # noqa: E501
        """Issues a loan payment.  # noqa: E501

        This endpoint issues a request to transfer funds from the consuming application's funding source to the beneficiary's loan account.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_payment_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param CreatePaymentRequest body: (required)
        :return: CreatePaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_payment" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `create_payment`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['apiKey']  # noqa: E501

        return self.api_client.call_api(
            '/payments', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='CreatePaymentResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_payment(self, uuid, **kwargs):  # noqa: E501
        """Check payment status.  # noqa: E501

        This endpoint retrieves an existing payment, updated with the latest status of its processing along with error information.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_payment(uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str uuid: (required)
        :return: CreatePaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_payment_with_http_info(uuid, **kwargs)  # noqa: E501
        else:
            (data) = self.get_payment_with_http_info(uuid, **kwargs)  # noqa: E501
            return data

    def get_payment_with_http_info(self, uuid, **kwargs):  # noqa: E501
        """Check payment status.  # noqa: E501

        This endpoint retrieves an existing payment, updated with the latest status of its processing along with error information.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_payment_with_http_info(uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str uuid: (required)
        :return: CreatePaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_payment" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'uuid' is set
        if ('uuid' not in params or
                params['uuid'] is None):
            raise ValueError("Missing the required parameter `uuid` when calling `get_payment`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'uuid' in params:
            path_params['uuid'] = params['uuid']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['apiKey']  # noqa: E501

        return self.api_client.call_api(
            '/payments/{uuid}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='CreatePaymentResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)
